package FkPPD;

use warnings;
use strict;
use Exporter;
use ActiveState::Handy qw(run);
use ActiveState::RelocateTree;
use MkPPD;
our @ISA = qw(Exporter MkPPD);
our @EXPORT_OK = qw(get_perlapp);

# fake ppd
#
# Utility functions for creating non-ppm packages.
# These packages create a fake ppd file to allow ppm to track 
# package versioning.  Tarballs can be created with AS::Install
# functionality.

# WriteFakePPD
# Create a ppd file suitable for packages that need special 
# installation.  The PPD files are only for versioning, and
# DO NOT provide installation capabilites for the package.
# This functionality is in pmx-setup
# 
# - PureMessage-Perl
# - PureMessage-Sendmail
# - PureMessage-Postfix
# - PureMessage-PostgreSQL
# - pmx-setup
# - PureMessage-Snapshot-Tools
 
# overloads MkPPD::write()
sub write {
    my $self = shift;
    my $ppd = "$self->{name}.ppd";
    open(my $fh, ">", $ppd) || die "Can't create $ppd: $!";
    print $fh <<EOT;
<SOFTPKG NAME="$self->{name}" VERSION="$self->{ppd_version}">
<TITLE>$self->{name}</TITLE>
<ABSTRACT>$self->{abstract}</ABSTRACT>
<AUTHOR>$self->{author}</AUTHOR>
EOT

    print $fh <<EOPPD;

<IMPLEMENTATION>
EOPPD

    # print dependencies
    push @{$self->{requires}}, MkPPD::parse_requires unless $self->{no_REQUIRES};
    if (my $req = $self->{requires}) {
        $req = [$req] unless ref($req);
        for (@$req) {
            my $v = 0;
            my $n = $_;
            ($n, $v) = ($1, $2) if /^(.*)-(\d+(\.\d+)*)$/;
            $v = MkPPD::_ppd_version($v);
            print $fh qq(<DEPENDENCY NAME="$n" VERSION="$v"/>\n);
        }
    }

    print $fh <<EOPPD;
<CODEBASE HREF=""/>

<INSTALL EXEC="PPM_PERL">
BEGIN { unlink __FILE__ } # Old ppm workaround
# Install script
require 5.006;
use strict;

unless(\$ENV{PPM_JUST_DO_IT}) {
    print &lt;&lt;EOT;

$self->{name} cannot be installed using PPM.  Please use
pmx-setup to update this package.

EOT
    exit -1;
}
</INSTALL>

</IMPLEMENTATION>

</SOFTPKG>
EOPPD

    close($fh) || die "Could not write $ppd: $!";
    print "# wrote $ppd\n";

    # Write the install files
    #my $install <<EOT;
#EOT
    
    return 1;
}

# overloads MkPPD::write()
sub finish {
    my $self = shift;

    $self->write;
    my $tarball = $self->tarball;
    die if $tarball->{finished};
    my $tmpdir   = $tarball->{tmpdir}   or die "tmpdir isn't defined!";
    my $basename = $tarball->{basename} or die "basename isn't defined!";
    run("cd $tmpdir && tar cvf ../$basename.tar .");
    if ($^O eq "linux" or $^O eq "freebsd" or $^O eq "darwin") {
        run("gzip -f9 $basename.tar");
    }
    else {
        run("compress $basename.tar");
    }
    run("rm -rf $tmpdir");
    $tarball->{finished}++;
}

sub spongedir {
    my $self = shift;
    return "/tmp" . ("/zzzzzzzzz" x 5) . '/' . ($self->{spongedir_suffix} || $self->{name});
}

sub set_spongedir_suffix {
    my ($self, $suffix) = @_;
    $self->{spongedir_suffix} = $suffix;
}

sub add_pmx_install {
    my ($self, %opts) = @_;
    return $self->add_install(%opts, output_file => 'install_pmx');
}

sub add_root_install {
    my ($self, %opts) = @_;
    return $self->add_install(%opts, output_file => 'install_root');
}

sub add_install {
    my ($self, %opts) = @_;
    my $subs = delete $opts{substitutions} || {};
    my $reloc_dir = delete $opts{reloc_dir} || "";
    $reloc_dir = "/$reloc_dir" if $reloc_dir;
    my $install_script = delete $opts{install_script} or
        die "install_script is mandatory!";
    my $perlapp_ver = delete $opts{perlapp_ver};
    my $perlapp_libs = delete $opts{perlapp_libs} || [];
    my $perlapp_extra_options = delete $opts{perlapp_extra_options} || [];
    my $install_file = delete $opts{output_file} || 'install';

    # Create install.sh
#    open(my $ish, ">install.sh") or die "Can't open install.sh: $!";
#    print $ish <<EOT;
##!/bin/sh
#
#exec ./install "\$@"
#EOT
#    close $ish or die "Can't write install.sh: $!";
#    $self->tarball->add("install.sh");

    # XXX XXX XXX
    # DO NOT EDIT THE NEXT 7 LINES WITHOUT ADJUSTING for the offset!!!
    # XXX XXX XXX
    my ($__FILE__, $__LINE__) = (__FILE__, __LINE__ + 6);
    my $install_pl = <<HEADER;
#!/usr/bin/perl -w
#line $__LINE__ "$__FILE__"
HEADER
    $install_pl .= <<'EOT';
use strict;

use ActiveState::Install qw(install installed
                            CFG_FORCE_OURS CFG_KEEP_THEIRS
                            FILE_ABANDON FILE_ABANDON_MODIFIED);
use ActiveState::Handy qw(run cat);
use ActiveState::Unix::Pw qw(groupadd useradd);

(my $progname = $0) =~ s,.*/,,;
my $prefix = shift || usage();
$prefix =~ s,/$,,;

sub usage {
    die "Usage: $progname <prefix>\n";
}

my $pkg = "%%PKG%%";
my $ver = "%%VER%%";
EOT
    $install_pl .= qq|my \$dir = "\$prefix$reloc_dir";\n|;
    $install_pl .= <<'EOT';
my $spongedir = "%%SPONGEDIR%%";
my $ppmspongedir = "%%PPMSPONGEDIR%%";

if ($progname eq 'install' or $progname eq 'install_root') {
    die "$progname must be run as 'root'.\n" if $> != 0;
}
die "Prefix is too long, can't install.\n" if length($dir) > length($spongedir);

if (defined &PerlApp::exe) {
    my $dir = PerlApp::exe();
    $dir =~ s,/[^/]+$,,;
    chdir($dir) || die "Can't chdir '$dir': $!";
}

for my $reloc_type (qw(bin text)) {
    my $reloc_file = "reloc-$reloc_type";
    next unless -e $reloc_file;
    my $reloc_func = "relocate_$reloc_type";
    my $placeholder = {
	bin => [$spongedir => $dir,
                $ppmspongedir => $dir],
	text => ["%%"."PREFIX%%" => $dir,
	         "%%"."PMXPREFIX%%" => $prefix,
		 $spongedir => $dir,
                 $ppmspongedir => $dir],
    }->{$reloc_type};
    if (open(my $f, "<", $reloc_file)) {
	while (<$f>) {
	    chomp;
	    my @tmp = @$placeholder;
	    my $reloc = 0;
	    while (@tmp) {
		my ($k, $v) = (shift(@tmp), shift(@tmp));
		no strict 'refs';
		$reloc += &$reloc_func($_, $k, $v);
	    }
	    warn "Nothing to relocate in $_.\n" unless $reloc;
	}
    }
    else {
	warn "Can't open $reloc_file: $!";
    }
}

sub relocate_bin {
    my($file, $old, $new) = @_;

    my $pad_len = length($old) - length($new);
    die if $pad_len < 0;
    my $pad = "\0" x $pad_len;

    my $content = cat($file) || die "Can't find $file";
    if ($content =~ s,\Q$old\E([^\0]*),$new$1$pad,g) {
	#warn "$file relocated\n";
	open(my $f, ">", $file) || die "Can't overwrite $file: $!";
	binmode($f);
	print $f $content;
	close($f);
	return 1;
    }
    0;
}

sub relocate_text {
    my($file, $old, $new) = @_;

    my $content = cat($file) || die "Can't find $file";
    if ($content =~ s,\Q$old,$new,g) {
	#warn "$file relocated\n";
	open(my $f, ">", $file) || die "Can't overwrite $file: $!";
	print $f $content;
	close($f);
	return 1;
    }
    0;
}

sub find_file {
    my $f = shift;
    for my $d (qw(/usr/sbin /usr/lib /usr/bin)) {
	if (-f "$d/$f" and ! -l "$d/$f") {
	    return "$d/$f";
	}
    }
    return;
}

sub hide_file {
    my $old = shift;
    my $new = shift;
    return if $old eq $new;
    if (rename($old, "$old.pmxsave")) {
        print "Renamed $old to $old.pmxsave\n";
	if (symlink($new, $old)) {
	    print "$old has been symlinked to $new\n";
	    return 1;
	}
    }
    else {
        print "Could not move $old to $old.pmxsave: $!\n";
	print "You may want to do this yourself later.\n";
    }
    return 0;
}

# start installing
mkdir($dir, 0755);            # for standalone install
mkdir("$prefix/etc", 0755);  # for standalone install

# Record whether this is the first time installing the package. Useful for
# POST_INSTALL scripts that want to behave differently on upgrades.
my $pkg_installed = installed(
    pkg => $pkg,
    etc => "$prefix/etc",
);

EOT
    # Add the custom install script
    $install_pl .= "#line 1 $install_script\n";
    open(my $inst, $install_script) or die "Can't open $install_script: $!";
    { 
        local $/ = undef;
        $install_pl .= <$inst>;
    }
    close $inst or die "Can't close $install_script: $!";

    # Add the default substitutions
    $subs->{SPONGEDIR} = $self->spongedir;
    $subs->{PPMSPONGEDIR} = ActiveState::RelocateTree::spongedir('ppm');
    $subs->{PKG} = $self->{name};
    $subs->{VER} = $self->{version};
    # now we can perform substitutions
    $install_pl =~ s/%%(\w+)%%/$subs->{$1} || $1/eg;
   
    # now write out the install.pl and perlapp 
    open(my $ipl, ">$install_file.pl") or die "Can't open $install_file.pl: $!";
    print $ipl $install_pl;
    close $ipl or die "Can't write $install_file.pl: $!";

    # set some default lib paths
    my %libs = map { $_ => 1 } @$perlapp_libs;
    $libs{"../support/lib"}++;
    $libs{"../ActiveState-Config/lib"}++;
    $libs{"../ActiveState-Utils/lib"}++;
    my $lib = join ":", keys %libs;
    my $inc = join " -I", keys %libs;
    my $eopts = join " ", @$perlapp_extra_options;
    $ENV{PMX_PREFIX_FILE} = "../PureMessage-Install/t/etc/puremessage-5";
    run("$^X -c -I$inc $install_file.pl");
    my $perlapp = get_perlapp(min_ver => $perlapp_ver);
    run("$perlapp --nologo --force --lib $lib $eopts $install_file.pl");
    $self->tarball->add($install_file);
    unlink $install_file or die "Couldn't unlink install: $!";
    unlink "$install_file.pl" or die "Couldn't unlink $install_file.pl: $!";
}

# determine a specific version of perlapp exists
sub get_perlapp {
    my %opts = @_;
    my $req = $opts{min_ver} || 404;

    my $perlapp = $ENV{PERLAPP} || "perlapp";
    my $perlapp_version = `$perlapp --version 2>&1`;
    die "Can't run '$perlapp'"
        unless $perlapp_version && $perlapp_version =~ /build\s+(\d+)/;
    die "You will need to build with a non-expiring license of perlapp"
        if $perlapp_version =~ /expire/i;
    my $perlapp_build = $1;
    warn "You may need a better perlapp (build $req advised)"
        unless $perlapp_build >= $req;
    return "$perlapp --perl $^X";
}

1;
