package PPM::PureMessage::UA;

use strict;
use base 'LWP::UserAgent';
use Config qw(%Config);
use POSIX;

my $serial = '';
my $pmx_version = '0.1';
my $as_engine_version = '';
my $hostname = '';
my $pmx_prefix = '';
my %ppm_info;

eval {
    require PureMessage::_sys;
    my %kv = PureMessage::_sys::lic_kv();
    if (exists $kv{SerialNo}) {
	$serial = $kv{SerialNo};
    }
};

eval {
    # Figure out if we're running from an installed PureMesssage
    require PureMessage::Version;
    $pmx_version = PureMessage::Version::Version();
    $pmx_version .= ".$PureMessage::Version::BUILD";
    $pmx_prefix = $PureMessage::Version::PREFIX;
};

eval {
    require Net::Domain;
    Net::Domain->import('hostfqdn');
    $hostname = hostfqdn() || '';
};


sub new {
    my $class = shift;
    my %opts = @_; 
    
    if ($pmx_prefix) {
        eval {
            # If we're running under perlapp, we don't have the local perl's
            # libraries in our search path, including ppm.xml.  This means we must
            # manually specify where to find the 'real' installed ppm.xml	
            $ENV{'PPM_DAT'} = "$pmx_prefix/lib/site_perl/$Config{version}/ppm.xml";

            require PPM;
            PPM::read_config();  # make sure %PPM::repositories is initialized
            
            %ppm_info = PPM::InstalledPackageProperties();
            $as_engine_version = $ppm_info{'PureMessage-AntiSpam-Engine'}->{VERSION};
            $as_engine_version =~ s/,/\./g;
            
            delete $ENV{'PPM_DAT'};
        };
    }

    $as_engine_version = "0.1" unless $as_engine_version;

    my (undef, undef, $sysrelease, undef, $sysmachine) = POSIX::uname();

    my $agent = 'pmx-ppm/' . $pmx_version;
    $agent .= ' as-engine-ppm/' . $as_engine_version;
    $agent .= ' (';
    $agent .= "$serial; ";
    $agent .= "$^O; $sysrelease, $sysmachine; ";
    $agent .= "$hostname" . ') ';

    $opts{timeout} = 180; # override any timeout that was passed!!! 

    return $class->SUPER::new(agent => $agent,
			      keep_alive => 1,
			      env_proxy => 1,
                  %opts,
                  );
}

sub prepare_request {
    my($self, $request) = @_;
    $request->header("X-PMX-SerialNo", $serial) if $serial;

    # legacy (better to set user:pass in the http_proxy URL);
    my $proxy_user = $ENV{http_proxy_user};
    $proxy_user = $ENV{HTTP_proxy_user} unless defined $proxy_user;
    my $proxy_pass = $ENV{http_proxy_pass};
    $proxy_pass = $ENV{HTTP_proxy_pass} unless defined $proxy_pass;
    if (defined $proxy_user && defined $proxy_pass) {
        $request->proxy_authorization_basic($proxy_user, $proxy_pass);
    }

    # We only have ppm_info when PPM has been installed on the local system
    if (%ppm_info) {
        # Do we need to do authorization?
        # This is a hack, but will have to do for now.
        my $uri = $request->uri;
        foreach (keys %PPM::repositories) {
            if ($uri =~ /^\Q$PPM::repositories{$_}{'LOCATION'}\E/i) {
                my $username = $PPM::repositories{$_}{'USERNAME'};
                my $password = $PPM::repositories{$_}{'PASSWORD'};
                if (defined $username && defined $password) {
                    $request->authorization_basic($username, $password);
                    last;
                }
            }
        }
    }
    #print $request->as_string;
    return $self->SUPER::prepare_request($request);
}

1;
