package Sophos::Cover;

use strict;
use File::Find;

my %pm;
my %keep = (
    ActiveState => 1,
    Sophos => 1,
    PureMessage => 1,
    PerlMx => 1,
);

find({
	wanted => sub {
	    if (/\.pm$/) {
		my @p = split m#/#;
		shift @p while @p and not $keep{$p[0]};
		$pm{join '/', '', @p} = 1 if @p;
	    }
	},
	no_chdir => 1,
}, '.');

require Devel::Cover;
use Cwd qw(realpath);

my @ignore;
for (@INC) {
    next if /^\.$/;
    next if /\bblib\b/;
    push @ignore, $_;
}

# exclude PM-Perl directory
push @ignore, realpath('..');

# exclude test directory
push @ignore, realpath('t') if -d 't';

# log environment details
my $envfile = "cover_db/env.txt";
if (-d "cover_db" and !-e $envfile) {
    open(my $fh, ">$envfile") or die "Can't open $envfile: $!";
    print $fh "Cover: -select: " . join(", ", keys %pm) . "\n",
              "Cover: -inc: " . join(", ", @ignore) . "\n",
              "\nEnv vars:\n  ",
              join("\n  ", map { "$_\t=> $ENV{$_}" } keys %ENV);
    close $fh or die "Can't write $envfile: $!";
}

Devel::Cover->import(
    -silent => 1,
    -select => keys(%pm),
    -inc => @ignore,
    -ignore => '/Sophos/Cover',
	       '\.\./',
	       '^t/',
	       '\.t$',
	       '^test\.pl$',
);

1;
