package Sophos::Config::Depend;

use strict;
use Sophos::Config;
use base qw(Exporter);

our @EXPORT_OK = qw(config_getdeps config_mtime_deps);

sub config_getdeps {
    my $file = shift;
    my $p = Sophos::Config->new;
    my @dep;
    $p->visit_includes($file, sub { push @dep, $_[0] });
    @dep;
}

sub config_mtime_deps {
    my $file = shift;
    my $mtime = 0;
    for my $f (config_getdeps($file)) {
	my $m = (stat $f)[9];
	next unless defined $m;
	$mtime = $m if $m > $mtime;
    }
    return $mtime;
}

1;

__END__

=head1 NAME

Sophos::Config::Depend - Find configuration file dependencies

=head1 SYNOPSIS

  use Sophos::Config::Depend qw(config_getdeps config_mtime_deps);

  my @dep   = config_getdeps("foo.conf");
  my $mtime = config_mtime_deps("foo.conf");

=head1 DESCRIPTION

This module can export the following functions:

=over

=item @deps = config_getdeps( $file )

This function return all dependencies for a given configuration file.
Dependencies are defined as files that will be included by the given
configuration file.

=item $mtime = config_mtime_deps( $file )

This function return the time that the configuration file or any of
its dependencies last were modified.

=back

=cut
