package Sophos::Config::Simple;

use strict;
use Sophos::Config;
use base qw(Exporter);

our @EXPORT_OK = qw(config_parse config_parse_file);

sub config_parse_file {
    my $file = shift;
    my $p = Sophos::Config->new;
    $p->parse_file($file);
    return $p->as_hashref;
}

sub config_parse {
    my $text = shift;
    my $p = Sophos::Config->new;
    $p->parse($text);
    return $p->as_hashref;
}

1;

__END__

=head1 NAME

Sophos::Config::Simple - Simple-minded configuration file parser

=head1 SYNOPSIS

  use Sophos::Config::Simple qw(config_parse_file);

  my $conf = config_parse_file("foo.conf");

=head1 DESCRIPTION

This module exports a single function called config_parse_file() that
parse the configuration file using the same rules as
Sophos::Config and return a hash reference similar to what
Sophos::Config's as_hash method returns.

Sophos::Config::Simple is not able to run validators on the
configuration values, but it runs faster.

=cut
